<?php
class ModelPaymentQuantum extends Model {
	public function install() {
		$this->db->query("
			CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "quantum_order` (
			  `quantum_order_id` int(11) NOT NULL AUTO_INCREMENT,
			  `order_id` int(11) NOT NULL,
			  `date_added` DATETIME NOT NULL,
			  `date_modified` DATETIME NOT NULL,
			  PRIMARY KEY (`quantum_order_id`)
			) ENGINE=MyISAM DEFAULT COLLATE=utf8_general_ci;");

		$this->db->query("
			CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "quantum_order_transaction` (
			  `quantum_order_transaction_id` int(11) NOT NULL AUTO_INCREMENT,
			  `quantum_order_id` int(11) NOT NULL,
			  `transaction_id` CHAR(20) NOT NULL,
			  `date_added` DATETIME NOT NULL,
			  `amount` DECIMAL( 10, 2 ) NOT NULL,
			  `trans_result` VARCHAR(20) NOT NULL,
			  `trans_type` VARCHAR(20) NOT NULL,
			  `customer_id` VARCHAR(255) NOT NULL,
			  `auth_code` VARCHAR(50) NOT NULL,
			  `md5_hash` VARCHAR(50) NOT NULL,
			  `avs` VARCHAR(255) NOT NULL,
			  `cvv2` VARCHAR(10) NOT NULL,
			  `last_4cc` int(11) NOT NULL,
			  PRIMARY KEY (`quantum_order_transaction_id`)
			) ENGINE=MyISAM DEFAULT COLLATE=utf8_general_ci;");

		$this->db->query("
			CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "quantum_customer` (
			  `quantum_customer_id` int(11) NOT NULL AUTO_INCREMENT,
			  `customer_id` int(11) NOT NULL,
			  `date_added` DATETIME NOT NULL,
			  PRIMARY KEY (`quantum_customer_id`)
			) ENGINE=MyISAM DEFAULT COLLATE=utf8_general_ci;");
	}

	public function uninstall() {
		$this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "quantum_order_transaction`;");
		$this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "quantum_order`;");
	}

	function assocArrayToXML($root_element_name,$ar) {
		$xml = new SimpleXMLElement("<?xml version=\"1.0\"?><{$root_element_name}></{$root_element_name}>");
		$f = create_function('$f,$c,$a','
            foreach($a as $k=>$v) {
                if(is_array($v)) {
                    $ch=$c->addChild($k);
                    $f($f,$ch,$v);
                } else {
                    $c->addChild($k,$v);
                }
            }');
		$f($f,$xml,$ar);
		return $xml->asXML();
	}

	function xmlToArray($xmlObject) {
		$result = array();
		foreach ((array) $xmlObject as $key => $value) {
			$result[$key] = (is_object($value)) ? $this->xmlToArray($value) : $value;
		}
		return $result;
	}

	public function call($data, $options = array('format' => 'json')) {
		$format = 'json';
		if ($options['format'] == 'array') {
			$format = $options['format'];
		}

		$api_endpoint = 'https://secure.quantumgateway.com/cgi/xml_requester.php';

		if ($data['request_type'] == 'ShowTransactionDetails') {
			$xmlData = array(
				'Authentication' => array(
					'GatewayLogin' => $data['gwlogin'],
					'GatewayKey' => $data['restrict_key'],
				),
				'Request' => array(
					'RequestType' => $data['request_type'],
					'TransactionID' => $data['transaction_id'],
				),
			);
		} else if ($data['request_type'] == 'ProcessSingleTransaction') {
			$xmlData = array(
				'Authentication' => array(
					'GatewayLogin' => $data['gwlogin'],
					'GatewayKey' => $data['restrict_key'],
				),
				'Request' => array(
					'RequestType' => $data['request_type'],
					'ProcessType' => $data['process_type'],
					'TransactionID' => $data['transaction_id'],
				),
			);
			if (isset($data['credit_card_number'])) {
				$xmlData['Request']['CreditCardNumber'] = $data['credit_card_number'];
			}
		} else {
			$xmlData = array (
				'Authentication' => array(
					'GatewayLogin' => $data['gwlogin'],
					'GatewayKey' => $data['restrict_key'],
				),
				'Request' => array(
					'RequestType' => $data['request_type'],
					'Status' => $data['status'],
					'TransactionDateBegin' => $data['date_begin'], //'2015-02-02',
					'TransactionDateEnd' => $data['date_end'], //'2015-02-05',
					'TransactionType' => $data['transaction_type'],
					'CustomerID' => isset($data['customer_id']) ? $data['customer_id'] : '',
					'PaymentType' => isset($data['payment_type']) ? $data['payment_type'] : '',
					'SettlementBatchID' => isset($data['settlement_batch_id']) ? $data['settlement_batch_id'] : '',
					'SettlementDateBegin' => isset($data['settlement_date_begin']) ? $data['settlement_date_begin'] : '',
					'SettlementDateEnd' => isset($data['settlement_date_end']) ? $data['settlement_date_end'] : '',
				),
			);
		}

		$xml = $this->assocArrayToXML('QGWRequest', $xmlData);

		$defaults = array(
			CURLOPT_POST => 1,
			CURLOPT_HEADER => 0,
			CURLOPT_URL => $api_endpoint,
			CURLOPT_USERAGENT => "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.1) Gecko/20061204 Firefox/2.0.0.1",
			CURLOPT_FRESH_CONNECT => 1,
			CURLOPT_RETURNTRANSFER => 1,
			CURLOPT_FORBID_REUSE => 1,
			CURLOPT_TIMEOUT => 0,
			CURLOPT_SSL_VERIFYPEER => 0,
			CURLOPT_SSL_VERIFYHOST => 0,
			CURLOPT_POSTFIELDS => "xml=" . $xml
		);

		$ch = curl_init();

		curl_setopt_array($ch, $defaults);

		if (!$result = curl_exec($ch)) {

			$log_data = array(
				'curl_error' => curl_error($ch),
				'curl_errno' => curl_errno($ch)
			);

			return false;
		}

		curl_close($ch);

		$result = str_replace(array("\n", "\r", "\t"), '', $result);
		$result =  trim(str_replace('"', "'", $result));
		$simpleXml = simplexml_load_string($result);

		if ($format == 'json') {
			$json = json_encode($simpleXml);
			return $json;
		} else if ($format == 'array') {
			return $this->xmlToArray($simpleXml);
		} else {
			return false;
		}
	}

	public function getTransactions($call_data) {
		$this->load->model('setting/setting');

		$call_data['request_type'] = 'SearchTransactions';
		$call_data['gwlogin'] = $this->config->get('quantum_gwlogin');
		$call_data['restrict_key'] = $this->config->get('quantum_restrict_key');

		return $this->call($call_data);
	}

	public function getTransaction($transaction_id) {
		$call_data = array(
			'request_type' => 'ShowTransactionDetails',
			'gwlogin' => $this->config->get('quantum_gwlogin'),
			'restrict_key' => $this->config->get('quantum_restrict_key'),
			'transaction_id' => $transaction_id,
		);

		return $this->call($call_data, array('format' => 'array'));
	}

	public function void($transaction_id) {
		$call_data = array(
			'request_type' => 'ProcessSingleTransaction',
			'gwlogin' => $this->config->get('quantum_gwlogin'),
			'restrict_key' => $this->config->get('quantum_restrict_key'),
			'transaction_id' => $transaction_id,
			'process_type' => 'VOID',
		);

		return $this->call($call_data, array('format' => 'array'));
	}

	public function forceAuth($transaction_id) {
		$call_data = array(
			'request_type' => 'ProcessSingleTransaction',
			'gwlogin' => $this->config->get('quantum_gwlogin'),
			'restrict_key' => $this->config->get('quantum_restrict_key'),
			'transaction_id' => $transaction_id,
			'process_type' => 'PREVIOUS_SALE',
		);

		return $this->call($call_data, array('format' => 'array'));
	}

	public function returnTransaction($transaction_id, $credit_card_number) {
		$call_data = array(
			'request_type' => 'ProcessSingleTransaction',
			'gwlogin' => $this->config->get('quantum_gwlogin'),
			'restrict_key' => $this->config->get('quantum_restrict_key'),
			'transaction_id' => $transaction_id,
			'credit_card_number' => $credit_card_number,
			'process_type' => 'RETURN',
		);

		return $this->call($call_data, array('format' => 'array'));
	}

}

