<?php
class ControllerPaymentQuantum extends Controller {
	public function index() {
		$this->language->load('payment/quantum');
		$this->load->model('payment/quantum');

		$data['button_confirm'] = $this->language->get('button_confirm');

		$data['action'] = HTTPS_SERVER.'index.php?route=payment/quantum/authenticate';

		$data['text_wait'] = $this->language->get('text_wait');
		$data['text_dont_do_anything'] = $this->language->get('text_dont_do_anything');
		$data['text_use_saved_data'] = $this->language->get('text_use_saved_data');
		$data['text_store_to_vault'] = $this->language->get('text_store_to_vault');
		$data['lastip'] = $_SERVER['REMOTE_ADDR']; // maybe escape this
		$data['return_success'] = HTTPS_SERVER.'index.php?route=payment/quantum/success';
		$data['return_failure'] = HTTPS_SERVER.'index.php?route=payment/quantum/failure';

		$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);

		$data['amount'] = $this->currency->format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false);
		$data['first_name'] = html_entity_decode($order_info['payment_firstname'], ENT_QUOTES, 'UTF-8');
		$data['last_name'] = html_entity_decode($order_info['payment_lastname'], ENT_QUOTES, 'UTF-8');
		$data['company'] = html_entity_decode($order_info['payment_company'], ENT_QUOTES, 'UTF-8');
		$data['address'] = html_entity_decode($order_info['payment_address_1'], ENT_QUOTES, 'UTF-8') . ' ' . html_entity_decode($order_info['payment_address_2'], ENT_QUOTES, 'UTF-8');
		$data['city'] = html_entity_decode($order_info['payment_city'], ENT_QUOTES, 'UTF-8');
		$data['state'] = html_entity_decode($order_info['payment_zone'], ENT_QUOTES, 'UTF-8');
		$data['zip'] = html_entity_decode($order_info['payment_postcode'], ENT_QUOTES, 'UTF-8');
		$data['country'] = html_entity_decode($order_info['payment_country'], ENT_QUOTES, 'UTF-8');
		$data['email'] = $order_info['email'];
		$data['phone'] = $order_info['telephone'];
		$data['trans_type'] = $this->config->get('quantum_trans_type');
		$data['override_trans_email'] = $this->config->get('quantum_override_trans_email');
		$data['override_email_customer'] = $this->config->get('quantum_override_email_customer');
		$data['shipping_first_name'] = html_entity_decode($order_info['shipping_firstname'], ENT_QUOTES, 'UTF-8');
		$data['shipping_last_name'] = html_entity_decode($order_info['shipping_lastname'], ENT_QUOTES, 'UTF-8');
		$data['shipping_address'] = html_entity_decode($order_info['shipping_address_1'], ENT_QUOTES, 'UTF-8');
		$data['shipping_city'] = html_entity_decode($order_info['shipping_city'], ENT_QUOTES, 'UTF-8');
		$data['shipping_state'] = html_entity_decode($order_info['shipping_zone'], ENT_QUOTES, 'UTF-8');
		$data['shipping_zip'] = html_entity_decode($order_info['shipping_postcode'], ENT_QUOTES, 'UTF-8');
		$data['shipping_country'] = html_entity_decode($order_info['shipping_country'], ENT_QUOTES, 'UTF-8');
				
		$data['order_id'] = $order_info['order_id'];

		$data['ssl_is_on'] = false;
		$data['quantum_existing_customer'] = false;
		if (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == "on") {
			$data['ssl_is_on'] = true;

			$customer_id = $order_info['customer_id'];
			$quantum_customer_id = false;
			if ($customer_id) {
				// 'REGISTERED CUSTOMER';
				$quantum_customer_id = $this->model_payment_quantum->getCustomer($customer_id);
				// We seem to have a Quantum customer ID
				if ($quantum_customer_id) {
					$data['quantum_existing_customer'] = true;
					$data['action'] = HTTPS_SERVER.'index.php?route=payment/quantum/process_transaction';
				}
				if (!$quantum_customer_id) {
					$quantum_customer_id = $this->model_payment_quantum->addCustomer($customer_id);
				}
			}

			$data['quantum_customer_id'] = $quantum_customer_id;
		} else {
			$data['ssl_link'] = HTTPS_SERVER . 'index.php?route=checkout/checkout&token=' . $this->session->data['token'];
		}


		if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/quantum.tpl')) {
			return $this->load->view($this->config->get('config_template') . '/template/payment/quantum.tpl', $data);
		} else {
			return $this->load->view('default/template/payment/quantum.tpl', $data);
		}
	}

	public function process_transaction() {
		$this->load->model('checkout/order');
		$this->load->model('payment/quantum');

		$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);
		$customer_id = $order_info['customer_id'];
		$order_id = $order_info['order_id'];

		$quantum_customer_id = $this->model_payment_quantum->getCustomer($customer_id);
		$amount = $this->currency->format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false);

		$result = $this->model_payment_quantum->createTransaction($amount, $quantum_customer_id, $order_id);

		$this->storeOrderData(array(
			'order_id' => $result['Result']['Memo'],
			'transaction_id' => $result['Result']['TransactionID'],
			'amount' => $result['Result']['Amount'],
		), array(
			'trans_result' => $result['Result']['Status'],
			//'trans_type'   => $this->request->get['trans_type'],
			'customer_id'  => $result['Result']['CustomerID'],
			//'md5_hash'     => $this->request->get['md5_hash'],
			'avs'          => $result['Result']['AVSResponseCode'],
			'cvv2'         => $result['Result']['CVV2ResponseCode'],
			'last_4cc'     => $result['Result']['CreditCardNumber'],
			'auth_code'    => $result['Result']['AuthorizationCode'],
		));

		if ($result['Result']['Status'] === 'APPROVED') {
			$this->response->redirect($this->url->link('checkout/success', '', 'SSL'));
		} else {
			$this->response->redirect($this->url->link('checkout/failure', '', 'SSL'));
		}
	}

	public function customer_edit() {
		if (!$this->customer->isLogged()) {
			$this->session->data['redirect'] = $this->url->link('account/account', '', 'SSL');

			$this->response->redirect($this->url->link('account/login', '', 'SSL'));
		}

		$this->load->language('account/account');
		$this->load->language('payment/quantum');

		$this->load->model('payment/quantum');
		$json = $this->model_payment_quantum->authenticate();

		$this->document->setTitle($this->language->get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_account'),
			'href' => $this->url->link('account/account', '', 'SSL')
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_customer_edit'),
			'href' => $this->url->link('payment/quantum/customer_edit', '', 'SSL')
		);

		if (isset($this->session->data['success'])) {
			$data['success'] = $this->session->data['success'];

			unset($this->session->data['success']);
		} else {
			$data['success'] = '';
		}

		$data['heading_title'] = $this->language->get('heading_title');

		$data['text_customer_edit'] = $this->language->get('text_customer_edit');
		$data['json'] = $json;
		$data['lastip'] = $_SERVER['REMOTE_ADDR']; // maybe escape this

		$data['return_success'] = HTTPS_SERVER.'index.php?route=payment/quantum/return_success_customer_edit';
		$data['return_failure'] = HTTPS_SERVER.'index.php?route=payment/quantum/return_failure_customer_edit';

		$customer_id = $this->customer->getId();
		$data['quantum_customer_id'] = $this->model_payment_quantum->getCustomer($customer_id);

		$data['column_left'] = $this->load->controller('common/column_left');
		$data['column_right'] = $this->load->controller('common/column_right');
		$data['content_top'] = $this->load->controller('common/content_top');
		$data['content_bottom'] = $this->load->controller('common/content_bottom');
		$data['footer'] = $this->load->controller('common/footer');
		$data['header'] = $this->load->controller('common/header');

		if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/quantum_customer_edit.tpl')) {
			$this->response->setOutput($this->load->view($this->config->get('config_template') . '/template/payment/quantum_customer_edit.tpl', $data));
		} else {
			$this->response->setOutput($this->load->view('default/template/payment/quantum_customer_edit.tpl', $data));
		}
	}

	public function return_success_customer_edit() {
		$data['return_success'] = HTTPS_SERVER.'index.php?route=payment/quantum/success_customer_edit';
		if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/quantum_success_customer_edit_return.tpl')) {
			$this->response->setOutput($this->load->view($this->config->get('config_template') . '/template/payment/quantum_success_customer_edit_return.tpl', $data));
		} else {
			$this->response->setOutput($this->load->view('default/template/payment/quantum_success_customer_edit_return.tpl', $data));
		}
	}

	public function return_failure_customer_edit() {
		$data['return_failure'] = HTTPS_SERVER.'index.php?route=payment/quantum/failure_customer_edit';
		if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/quantum_failure_customer_edit_return.tpl')) {
			$this->response->setOutput($this->load->view($this->config->get('config_template') . '/template/payment/quantum_failure_customer_edit_return.tpl', $data));
		} else {
			$this->response->setOutput($this->load->view('default/template/payment/quantum_failure_customer_edit_return.tpl', $data));
		}
	}

	public function success_customer_edit() {
		if (!$this->customer->isLogged()) {
			$this->session->data['redirect'] = $this->url->link('account/account', '', 'SSL');

			$this->response->redirect($this->url->link('account/login', '', 'SSL'));
		}

		$this->load->language('account/account');
		$this->load->language('payment/quantum');

		$this->load->model('payment/quantum');

		$this->document->setTitle($this->language->get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_account'),
			'href' => $this->url->link('account/account', '', 'SSL')
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_customer_edit'),
			'href' => $this->url->link('payment/quantum/customer_edit', '', 'SSL')
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_success'),
			'href' => $this->url->link('payment/quantum/success_customer_edit', '', 'SSL')
		);

		if (isset($this->session->data['success'])) {
			$data['success'] = $this->session->data['success'];

			unset($this->session->data['success']);
		} else {
			$data['success'] = '';
		}

		$data['heading_title'] = $this->language->get('heading_title');

		$data['text_customer_edit'] = $this->language->get('text_success');
		$data['text_customer_edit_success'] = $this->language->get('text_customer_edit_success');

		$data['text_go_back'] = $this->language->get('text_go_back');
		$data['link_go_back'] = $this->url->link('account/account', '', 'SSL');

		$data['column_left'] = $this->load->controller('common/column_left');
		$data['column_right'] = $this->load->controller('common/column_right');
		$data['content_top'] = $this->load->controller('common/content_top');
		$data['content_bottom'] = $this->load->controller('common/content_bottom');
		$data['footer'] = $this->load->controller('common/footer');
		$data['header'] = $this->load->controller('common/header');
		if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/quantum_success_customer_edit.tpl')) {
			$this->response->setOutput($this->load->view($this->config->get('config_template') . '/template/payment/quantum_success_customer_edit.tpl', $data));
		} else {
			$this->response->setOutput($this->load->view('default/template/payment/quantum_success_customer_edit.tpl', $data));
		}
	}

	public function failure_customer_edit() {
		if (!$this->customer->isLogged()) {
			$this->session->data['redirect'] = $this->url->link('account/account', '', 'SSL');

			$this->response->redirect($this->url->link('account/login', '', 'SSL'));
		}

		$this->load->language('account/account');
		$this->load->language('payment/quantum');

		$this->load->model('payment/quantum');

		$this->document->setTitle($this->language->get('heading_title'));

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/home')
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_account'),
			'href' => $this->url->link('account/account', '', 'SSL')
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_customer_edit'),
			'href' => $this->url->link('payment/quantum/customer_edit', '', 'SSL')
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_failure'),
			'href' => $this->url->link('payment/quantum/failure_customer_edit', '', 'SSL')
		);

		if (isset($this->session->data['success'])) {
			$data['success'] = $this->session->data['success'];

			unset($this->session->data['success']);
		} else {
			$data['success'] = '';
		}

		$data['heading_title'] = $this->language->get('heading_title');

		$data['text_customer_edit'] = $this->language->get('text_failure');
		$data['text_customer_edit_failure'] = $this->language->get('text_customer_edit_failure');

		$data['text_go_back'] = $this->language->get('text_go_back');
		$data['link_go_back'] = $this->url->link('account/account', '', 'SSL');

		$data['column_left'] = $this->load->controller('common/column_left');
		$data['column_right'] = $this->load->controller('common/column_right');
		$data['content_top'] = $this->load->controller('common/content_top');
		$data['content_bottom'] = $this->load->controller('common/content_bottom');
		$data['footer'] = $this->load->controller('common/footer');
		$data['header'] = $this->load->controller('common/header');
		if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/quantum_failure_customer_edit.tpl')) {
			$this->response->setOutput($this->load->view($this->config->get('config_template') . '/template/payment/quantum_failure_customer_edit.tpl', $data));
		} else {
			$this->response->setOutput($this->load->view('default/template/payment/quantum_failure_customer_edit.tpl', $data));
		}
	}

	public function success() {
		$md5_key = $this->config->get('quantum_md5_key');
		$gwLogin = $this->config->get('quantum_gwlogin');
		$transID = isset($_GET['transID']) ? $this->request->get['transID'] : false;
		$amount = isset($_GET['Amount']) ? $this->request->get['Amount'] : false;
		$order_id = isset($_GET['ID']) ? $this->request->get['ID'] : false;
		$hash =  $md5_key.$gwLogin.$transID.$amount;
		$md5_hash = md5($hash);

		if (isset($_GET['md5_hash']) && $_GET['md5_hash'] == $md5_hash) {
			$this->storeOrderData(array(
				'order_id' => $order_id,
				'transaction_id' => $transID,
				'amount' => $amount,
			), array(
				'trans_result' => $this->request->get['trans_result'],
				'trans_type'   => $this->request->get['trans_type'],
				'customer_id'  => $this->request->get['CustomerID'],
				'md5_hash'     => $this->request->get['md5_hash'],
				'avs'          => $this->request->get['AVSResponse'],
				'cvv2'         => $this->request->get['CVV2Response'],
				'last_4cc'     => $this->request->get['Last4CC'],
				'auth_code'    => $this->request->get['authCode'],
			));
		} else {
			// something went wrong, maybe malicious attempt was made
		}

		// load success.tpl
		echo $this->load->view('default/template/payment/quantum_success.tpl');
	}

	public function failure() {
		// do same as success method but load failure.tpl
		echo $this->load->view('default/template/payment/quantum_failure.tpl');
	}

	public function storeOrderData($order_data, $transaction_data) {
		$this->load->model('payment/quantum');
		$this->load->model('checkout/order');

		$quantum_order_data = array(
			'order_id'         => $order_data['order_id'],
			'capture_status'   => 'capture_status_here'
		);

		$order_status_id = 5;   // 5 == Complete

		$quantum_order_id = $this->model_payment_quantum->addOrder($quantum_order_data);

		$quantum_transaction_data = array(
			'quantum_order_id'       => $quantum_order_id,
			'transaction_id'         => $order_data['transaction_id'],
			'trans_result'           => $transaction_data['trans_result'],
			'trans_type'             => isset($transaction_data['trans_type']) ? $transaction_data['trans_type'] : '',
			'amount'                 => $order_data['amount'],
			'customer_id'            => $transaction_data['customer_id'],
			'md5_hash'               => isset($transaction_data['md5_hash']) ? $transaction_data['md5_hash'] : '',
			'avs'                    => $transaction_data['avs'],
			'cvv2'                   => $transaction_data['cvv2'],
			'last_4cc'               => $transaction_data['last_4cc'],
			'auth_code'              => $transaction_data['auth_code'],
		);

		$comment = 'Quantum Transaction ID: '.$order_data['transaction_id']."\nStatus: ".$transaction_data['trans_result'];

		$this->model_payment_quantum->addTransaction($quantum_transaction_data);

		$this->model_checkout_order->addOrderHistory($order_data['order_id'], $order_status_id, $comment);
	}

	public function authenticate() {

		$this->load->model('payment/quantum');

		$json = $this->model_payment_quantum->authenticate();

		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}
}
